@@ -3,19 +3,19 @@ package Crypt::OpenSSL::RSA;
 use strict;
 use Carp;
 
-use vars qw( $VERSION @ISA @EXPORT @EXPORT_OK $AUTOLOAD
+use vars qw ($VERSION @ISA @EXPORT @EXPORT_OK $AUTOLOAD
              $RSA_PKCS1_PADDING $RSA_SSLV23_PADDING $RSA_NO_PADDING
-             $RSA_PKCS1_OAEP_PADDING );
+             $RSA_PKCS1_OAEP_PADDING);
 
 require Exporter;
 require DynaLoader;
 require AutoLoader;
 
 @ISA = qw(Exporter DynaLoader);
-@EXPORT = qw( $RSA_PKCS1_PADDING $RSA_SSLV23_PADDING $RSA_NO_PADDING
-              $RSA_PKCS1_OAEP_PADDING );
+@EXPORT = qw($RSA_PKCS1_PADDING $RSA_SSLV23_PADDING $RSA_NO_PADDING
+             $RSA_PKCS1_OAEP_PADDING);
 
-$VERSION = '0.19';
+$VERSION = '0.20';
 
 bootstrap Crypt::OpenSSL::RSA $VERSION;
 
@@ -51,15 +51,18 @@ Crypt::OpenSSL::RSA - RSA encoding and decoding, using the openSSL libraries
   $rsa_priv = Crypt::OpenSSL::RSA->new_private_key($key_string);
   $plaintext = $rsa->encrypt($ciphertext);
 
-  $rsa = Crypt::OpenSSL::RSA->generate_key( 1024 ); # or
-  $rsa = Crypt::OpenSSL::RSA->generate_key( 1024, $prime );
+  $rsa = Crypt::OpenSSL::RSA->generate_key(1024); # or
+  $rsa = Crypt::OpenSSL::RSA->generate_key(1024, $prime);
 
   print "private key is:\n", $rsa->get_private_key_string();
-  print "public key is:\n", $rsa->get_public_key_string();
+  print "public key (in PKCS1 format) is:\n",
+        $rsa->get_public_key_string();
+  print "public key (in X509 format) is:\n",
+        $rsa->get_public_key_x509_string();
 
   $rsa_priv->use_md5_hash(); # use_sha1_hash is the default
   $signature = $rsa_priv->sign($plaintext);
-  print "Signed correctly\n" if ( $rsa->verify($plaintext, $signature) );
+  print "Signed correctly\n" if ($rsa->verify($plaintext, $signature));
 
 =head1 DESCRIPTION
 
@@ -79,23 +82,28 @@ this (never documented) behavior is no longer the case.
 =item new_public_key
 
 Create a new Crypt::OpenSSL::RSA object by loading a public key in
-from an X509 encoded string.  The string should include the
------BEGIN...----- and -----END...----- lines.  The padding is set to
-PKCS1_OAEP, but can be changed with the use_xxx_padding methods
+from a string containing Base64/DER-encoding of either the PKCS1 or
+X.509 representation of the key.  The string should include the
+-----BEGIN...----- and -----END...----- lines.
+
+The padding is set to PKCS1_OAEP, but can be changed with the
+use_xxx_padding methods
 
 =cut
 
 sub new_public_key
 {
-    my $self = shift->_new();
-    $self->load_public_key( @_ );
+    my ($proto, $p_string) = @_;
+    my $self = $proto->_new();
+    $self->load_public_key($p_string);
     return $self;
 }
 
 =item new_private_key
 
 Create a new Crypt::OpenSSL::RSA object by loading a private key in
-from an X509 encoded string.  The string should include the
+from an string containing the Base64/DER encoding of the PKCS1
+representation of the key.  The string should include the
 -----BEGIN...----- and -----END...----- lines.  The padding is set to
 PKCS1_OAEP, but can be changed with use_xxx_padding.
 
@@ -104,7 +112,7 @@ PKCS1_OAEP, but can be changed with use_xxx_padding.
 sub new_private_key
 {
     my $self = shift->_new();
-    $self->load_private_key( @_ );
+    $self->load_private_key(@_);
     return $self;
 }
 
@@ -123,16 +131,16 @@ I<NOTE> - using generate_key as an instance method on an rsa object
 
 sub generate_key
 {
-    if( ref $_[0] )
+    if (ref $_[0])
     {
         warn "use of generate_key as an instance method instead of as a constructor has been deprecated.";
         my $self = shift;
-        $self->_generate_key( @_ );
+        $self->_generate_key(@_);
     }
     else
     {
         my $self = shift->_new();
-        $self->_generate_key( @_ );
+        $self->_generate_key(@_);
         return $self;
     }
 }
@@ -151,9 +159,9 @@ computation.
 
 sub new_key_from_parameters
 {
-    my( $proto, $n, $e, $d, $p, $q ) = @_;
+    my($proto, $n, $e, $d, $p, $q) = @_;
     return $proto->_new_key_from_parameters
-        ( map { $_ ? $_->pointer_copy() : 0 } $n, $e, $d, $p, $q );
+        (map { $_ ? $_->pointer_copy() : 0 } $n, $e, $d, $p, $q);
 }
 
 =item import_random_seed
@@ -166,9 +174,9 @@ modules.
 
 sub import_random_seed
 {
-    until ( _random_status() )
+    until (_random_status())
     {
-        _random_seed( Crypt::OpenSSL::Random::random_bytes(20) );
+        _random_seed(Crypt::OpenSSL::Random::random_bytes(20));
     }
 }
 
@@ -221,46 +229,48 @@ I<DEPRECATED> - use new_public_key instead
 
 sub load_public_key
 {
-    my($self, $key_string) = @_;
-    $self->_load_key(0, $key_string);
+    my ($self, $p_key_string) = @_;
+    if ($p_key_string =~ /^-----BEGIN RSA PUBLIC KEY-----/)
+    {
+        $self->_load_public_pkcs1_key($p_key_string);
+    }
+    elsif ($p_key_string =~ /^-----BEGIN PUBLIC KEY-----/)
+    {
+        $self->_load_public_x509_key($p_key_string);
+    }
+    else
+    {
+        croak "unrecognized key format";
+    }
 }
 
 =item load_private_key
 
 I<DEPRECATED> - use new_private_key instead
 
-=cut
-
-sub load_private_key
-{
-    my($self, $key_string) = @_;
-    $self->_load_key(1, $key_string);
-}
-
 =item get_public_key_string
 
-Return the public portion of the key as an X509 encoded string.
+Return the Base64/DER-encoded PKCS1 representation of the public
+key.  This string has
+header and footer lines:
 
-=cut
+  -----BEGIN RSA PUBLIC KEY------
+  -----END RSA PUBLIC KEY------
 
-sub get_public_key_string
-{
-    my ($self) = @_;
-    return $self->_get_key_string(0);
-}
+=item get_public_key_x509_string
+
+Return the Base64/DER-encoded representation of the "subject
+public key", suitable for use in X509 certificates.  This string has
+header and footer lines:
+
+  -----BEGIN PUBLIC KEY------
+  -----END PUBLIC KEY------
+
+and is the format that is produced by running C<openssl rsa -pubout>.
 
 =item get_private_key_string
 
-Return the X509 encoding of the private key.
-
-=cut
-
-sub get_private_key_string
-{
-    my ($self) = @_;
-    return $self->_get_key_string(1);
-}
-
+Return the DER-encoded PKCS1 representation of the private key.
 
 =item encrypt
 
@@ -288,7 +298,7 @@ sub set_padding_mode
 {
     my $self = shift;
     warn "set_padding_mode is deprecated.  Use use_xxx_padding instead";
-    shift->_set_padding_mode( @_ ) ;
+    shift->_set_padding_mode(@_) ;
 }
 
 sub _set_padding_mode
@@ -323,7 +333,7 @@ denotes that the server is SSL3 capable.
 
 sub use_sslv23_padding
 {
-    shift->_set_padding_mode( $RSA_SSLV23_PADDING );
+    shift->_set_padding_mode($RSA_SSLV23_PADDING);
 }
 
 =item get_padding_mode
@@ -405,7 +415,8 @@ There is a small memory leak when generating new keys of more than 512 bits.
 
 =head1 AUTHOR
 
-Ian Robertson, iroberts@cpan.org
+Ian Robertson, iroberts@cpan.org.  For support, please email
+perl-openssl-users@lists.sourceforge.net.
 
 =head1 SEE ALSO
 
